@@ -4,8 +4,8 @@
 
 ;; Author: Shariff AM Faleel
 ;; Package-Requires: ((emacs "28") (org-roam "2.2.0") (s "1.12.0") (magit-section "3.3.0") (transient "0.4") (org-super-agenda "1.2") (dash "2.0"))
-;; Package-Version: 20251021.559
-;; Package-Revision: a207462ec885
+;; Package-Version: 20251116.829
+;; Package-Revision: b1b91d6e1847
 ;; Homepage: https://github.com/ahmed-shariff/org-roam-ql
 ;; SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -92,6 +92,14 @@ applied in order of appearance in the list."
 (defvar-local org-roam-ql--buffer-displayed-query nil "The query which produced the results of the buffer.")
 (defvar-local org-roam-ql-buffer-in nil
   "Define which option to use - `in-buffer' or `org-roam-db'.")
+;;;###autoload
+(defvar-local org-roam-ql--filter-for-roam nil "Filter that will be applied to org-roam section.
+
+This is expected to be a valid `org-roam-ql' query or nil.
+See `org-roam-backlinks-section-with-ql-filter' and `org-roam-reflinks-section-with-ql-filter'")
+
+;; See `kill-all-local-variables'
+(put 'org-roam-ql--filter-for-roam 'permanent-local t)
 
 ;; `bookmark.el' setup
 (defvar bookmark-make-record-function)
@@ -120,6 +128,7 @@ SOURCE-OR-QUERY can be one of the following:
   :where (= todo \"TODO\")], you can omit the part till after
   :where i.e., pass only [(= todo \"TODO\")] and the rest will get
   appended in the front.
+- A sql statement - i.e, a string starting with \"select\"
 - A list of org-roam-nodes or an org-roam-node.
 - A function that returns a list of org-roam-nodes.
 
@@ -152,6 +161,10 @@ functions with `org-roam-ql-register-sort-fn'."
                               query
                             (vconcat [:select id :from nodes :where] query))
                           args))))
+         ((and str
+               (pred stringp)
+               (guard (string-prefix-p "select" str t)))
+          (--map (org-roam-node-from-id (car it)) (org-roam-db-query str)))
          ((and (pred listp) (pred org-roam-ql--check-if-valid-query))
           (org-roam-ql--expand-query source-or-query))
          ((pred functionp)
@@ -603,7 +616,7 @@ Sets the history as well."
                                       (when (derived-mode-p 'org-roam-mode)
                                         (propertize (buffer-name)
                                                     'type 'buffer
-                                                    'data (if (equal (buffer-name) org-roam-buffer)
+                                                    'data (if (org-roam-ql--check-if-roam-buffer)
                                                               (cons org-roam-buffer "org-roam-buffer")
                                                             (with-current-buffer it
                                                               (cons org-roam-ql-buffer-query org-roam-ql-buffer-title))))))
@@ -976,7 +989,7 @@ Similar to `org-roam-mode', but doesn't default to the
 
 (defun org-roam-ql--refresh-roam-buffer ()
   "Refresh a org-roam buffer."
-  (if (equal (buffer-name) org-roam-buffer)
+  (if (org-roam-ql--check-if-roam-buffer)
       (if (not org-roam-ql-buffer-query)
           (org-roam-buffer-refresh)
         (let ((query org-roam-ql-buffer-query)
@@ -1043,13 +1056,14 @@ all values in `org-roam-ql-preview-postprocess-functions'."
           (oset section node previewing-node)
           (magit-insert-section section (org-roam-preview-section)
             ;; respecting buffer local value
-            (let ((preview-fn org-roam-ql-preview-function))
+            (let ((preview-fn org-roam-ql-preview-function)
+                  (query org-roam-ql-buffer-query))
               (insert (org-roam-fontify-like-in-org-mode
                        (save-excursion
                          (org-roam-with-temp-buffer (org-roam-node-file previewing-node)
                            (org-with-wide-buffer
                             (goto-char (org-roam-node-point previewing-node))
-                            (let ((s (funcall preview-fn previewing-node org-roam-ql-buffer-query)))
+                            (let ((s (funcall preview-fn previewing-node query)))
                               (dolist (fn org-roam-ql-preview-postprocess-functions)
                                 (setq s (funcall fn s)))
                               s)))))
@@ -1108,7 +1122,7 @@ See `org-roam-ql-search' for PREVIEW-FN."
 
 See docs of `bookmark-make-record-function'."
   (let ((bookmark (cons nil (bookmark-make-record-default 'no-file 'no-context))))
-    (if (equal (buffer-name) org-roam-buffer)
+    (if (org-roam-ql--check-if-roam-buffer)
         (progn
           (bookmark-prop-set bookmark 'handler #'org-roam-ql--bookmark-open)
           (bookmark-prop-set bookmark 'title
@@ -1288,6 +1302,86 @@ If there are entries that do not have an ID, it will signal an error"
                                         `(in-buffer ,(buffer-name)))
                                       org-roam-ql-preview-function))
 
+;; *****************************************************************************
+;; `org-roam' sections (`org-roam-mode-sections') that can also be
+;; filtered with org-roam-ql
+;; *****************************************************************************
+(cl-defun org-roam-backlinks-section-with-ql-filter (node &key (unique nil) (show-backlink-p nil)
+                                                          (section-heading "Backlinks:"))
+  "Same as `org-roam-backlinks-section', but considers the filter (`org-roam-ql--filter-for-roam') of the current buffer.
+
+When UNIQUE is nil, show all positions where references are found.
+When UNIQUE is t, limit to unique sources.
+
+When SHOW-BACKLINK-P is not null, only show backlinks for which
+this predicate is not nil.
+
+SECTION-HEADING is the string used as a heading for the backlink section."
+  (let* ((backlinks (seq-sort #'org-roam-backlinks-sort (org-roam-backlinks-get node :unique unique)))
+         (filter org-roam-ql--filter-for-roam)
+         (filter-node-ids (and filter
+                               (condition-case _err
+                                   (-map #'org-roam-node-id (org-roam-ql-nodes filter))
+                                 (user-error
+                                  (message "Cannot apply filter: %s" _err))))))
+    (when backlinks
+      (if filter-node-ids
+          (setq backlinks
+                (--filter (member (org-roam-node-id (org-roam-backlink-source-node it)) filter-node-ids)
+                          backlinks))
+        (setq filter nil))
+      (magit-insert-section (org-roam-backlinks)
+        (magit-insert-heading (if filter
+                                  (let ((has-colon (string-suffix-p ":" section-heading))
+                                        (filter-string (format "%S" filter)))
+                                    (concat (string-trim section-heading nil ":")
+                                            " (filter: "
+                                            (substring filter-string 0 (min 40 (length filter-string)))
+                                            ")"
+                                            (when has-colon ":")))
+                                section-heading))
+        (dolist (backlink backlinks)
+          (when (or (null show-backlink-p)
+                    (and (not (null show-backlink-p))
+                         (funcall show-backlink-p backlink)))
+            (org-roam-node-insert-section
+             :source-node (org-roam-backlink-source-node backlink)
+             :point (org-roam-backlink-point backlink)
+             :properties (org-roam-backlink-properties backlink))))
+        (insert ?\n)))))
+
+(defun org-roam-reflinks-section-with-ql-filter (node)
+  "The reflinks section for NODE with org-roam-ql filter (`org-roam-ql--filter-for-roam').
+
+Same as `org-roam-reflinks-section'."
+  (when-let ((refs (org-roam-node-refs node))
+             (reflinks (seq-sort #'org-roam-reflinks-sort (org-roam-reflinks-get node))))
+    (let* ((filter org-roam-ql--filter-for-roam)
+           (filter-node-ids (and filter
+                                 (condition-case _err
+                                     (-map #'org-roam-node-id (org-roam-ql-nodes filter))
+                                   (user-error
+                                    (message "Cannot apply filter: %s" _err))))))
+      (if filter-node-ids
+          (setq reflinks
+                (--filter (member (org-roam-node-id (org-roam-reflink-source-node it)) filter-node-ids)
+                          reflinks))
+        (setq filter nil))
+      (magit-insert-section (org-roam-reflinks)
+        (magit-insert-heading (if filter
+                                  (concat "Reflinks (filter:"
+                                          (-->
+                                           (format "%S" filter)
+                                           (substring it 0 (min 40 (length it))))
+                                          "):")
+                                "Reflinks:")
+          (dolist (reflink reflinks)
+            (org-roam-node-insert-section
+             :source-node (org-roam-reflink-source-node reflink)
+             :point (org-roam-reflink-point reflink)
+             :properties (org-roam-reflink-properties reflink)))
+          (insert ?\n))))))
+
 ;; *****************************************************************************
 ;; org-roam-ql transient
 ;; *****************************************************************************
@@ -1356,11 +1450,17 @@ If there are entries that do not have an ID, it will signal an error"
 
 (transient-define-prefix org-roam-ql-buffer-dispatch ()
   "Show `org-roam-ql' dispatcher."
-  ["Edit"
-   [("t" org-roam-ql-view--transient-title)
+  [[(:info
+    (lambda () (if (derived-mode-p 'org-roam-ql-mode)
+                   "Edit"
+                 "Query on org-roam-buffer"))
+    :format "%d" :face transient-heading)
+    ("t" org-roam-ql-view--transient-title)
     ("q" org-roam-ql-view--transient-query)
     ("s" org-roam-ql-view--transient-sort)
     ("i" org-roam-ql-view--transient-in)]]
+  [["Filter org-roam-buffer"
+    ("F" org-roam-ql-view--filter-roam)]]
   ["View"
    [("r" "Refresh" org-roam-ql-refresh-buffer)]
    [:if-derived org-roam-mode
@@ -1434,6 +1534,17 @@ If there are entries that do not have an ID, it will signal an error"
   :always-read t
   :choices '("in-buffer" "org-roam-db"))
 
+(transient-define-infix org-roam-ql-view--filter-roam ()
+  :class 'org-roam-ql--variable--sexp
+  :argument ""
+  :variable 'org-roam-ql--filter-for-roam
+  :prompt "Filter: "
+  :always-read t
+  :if (lambda () (org-roam-ql--check-if-roam-buffer))
+  :reader (lambda (&rest _)
+            (org-roam-ql--read-query (when org-roam-ql--filter-for-roam
+                                       (format "%S" org-roam-ql--filter-for-roam)))))
+
 ;; *****************************************************************************
 ;; org dynamic block
 ;; *****************************************************************************
@@ -1603,6 +1714,10 @@ Can be used in the minibuffer or when writting querries."
 Simple wrapper `org-roam-preview-function'."
   (funcall org-roam-preview-function))
 
+(defun org-roam-ql--check-if-roam-buffer ()
+  "Check if the buffer is a org-roam buffer but not a org-roam-ql buffer."
+  (and (derived-mode-p 'org-roam-mode) (not (derived-mode-p 'org-roam-ql-mode))))
+
 ;;;###autoload
 (defun org-roam-ql-convert-roam-buffer-to-roam-ql-buffer ()
   "Convert a `org-roam-mode' buffer to a `org-roam-ql-mode' buffer."
